"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Context_1 = require("../ioc/Context");
const LogManager_1 = require("../log/LogManager");
const PreinstantiatedSingletonDefinition_1 = require("../ioc/objectdefinition/PreinstantiatedSingletonDefinition");
const Lifecycle_1 = require("../ioc/Lifecycle");
const ConfigProvider_1 = require("../config/ConfigProvider");
class BaseApp {
    /**
     * Creates a new Inceptum App
     */
    constructor(options = {}) {
        this.plugins = [];
        this.pluginContext = new Map();
        LogManager_1.LogManager.setAppName(this.appName);
        const { config = new ConfigProvider_1.default(), logger = LogManager_1.LogManager.getLogger() } = options;
        this.logger = logger;
        this.context = new Context_1.Context(config.getConfig('app.context.name', 'BaseContext'), null, options);
        this.context.registerDefinition(new PreinstantiatedSingletonDefinition_1.PreinstantiatedSingletonDefinition(LogManager_1.LogManager));
    }
    use(...plugins) {
        return this.register(...plugins);
    }
    addDirectory(path) {
        return this.getContext().registerSingletonsInDir(path);
    }
    register(...plugins) {
        if (this.context.getStatus() !== Lifecycle_1.LifecycleState.NOT_STARTED) {
            throw new Error(`Cannot register plugin(s) ${plugins
                .map((p) => p.name)
                .join(',')} as the app has already started. Please register all plugins before calling "start()"`);
        }
        this.plugins = this.plugins.concat(plugins);
    }
    runLifecycleMethodOnPlugins(method) {
        return this.plugins.reduce(async (previous, plugin) => {
            await previous;
            if (plugin[method]) {
                this.logger.debug(`${method}:${plugin.name}`);
                return plugin[method](this, this.pluginContext);
            }
            return Promise.resolve();
        }, Promise.resolve());
    }
    async start() {
        await this.runLifecycleMethodOnPlugins('willStart');
        process.on('SIGINT', () => {
            this.stop().then(() => process.exit());
        });
        await this.context.lcStart();
        return await this.runLifecycleMethodOnPlugins('didStart');
    }
    async stop() {
        await this.runLifecycleMethodOnPlugins('willStop');
        this.logger.info('Shutting down app');
        await this.context.lcStop();
        return await this.runLifecycleMethodOnPlugins('didStop');
    }
    getContext() {
        return this.context;
    }
    // tslint:disable-next-line:prefer-function-over-method
    getConfig(key, defaultValue) {
        return this.getContext().getConfig(key, defaultValue);
    }
    // tslint:disable-next-line:prefer-function-over-method
    hasConfig(key) {
        return this.getContext().hasConfig(key);
    }
}
exports.default = BaseApp;
//# sourceMappingURL=BaseApp.js.map